#include "pch.h"
#include "CfgParse.hpp"

CCfgParser::CCfgParser() {
    AtomsNumber = 0;
    Line        = 0;
    Error       = 0;
    Opened      = 0;
}

CCfgParser::CCfgParser(char *arg_fname) {
    AtomsNumber = 0;
    Line        = 0;
    if (access(arg_fname, 0) == 0) {
        in_stream.open( arg_fname );
        if (in_stream) {
            Opened = 1;
            Error = 0;
        }
        else {
            Opened = 0;
            Error = 1;
        }
    }
    else {
        Error = 1;
        Opened = 0;
    }
}

int CCfgParser::Open(char *arg_fname) {
    if (Opened) {
        in_stream.close();
        FreeMem();
    }
    AtomsNumber = 0;
    Line        = 0;
    if (access(arg_fname, 0) == 0) {
        in_stream.open( arg_fname );
        if (in_stream) {
            Opened = 1;
            Error  = 0;
        }
        else {
            Opened = 0;
            Error  = 1;
        }
    }
    else {
        Opened = 0;
        Error  = 1;
    }
    return (Opened);
}

CCfgParser::~CCfgParser() {
    if (Opened)
        in_stream.close();
    FreeMem();
}

void CCfgParser::ReadLine() {
    FreeMem();
    in_stream.getline( buf, LINE_SIZE );
    Error = (in_stream == 0);
    if (!Error) {
        Line++;
        ParseLine();
    }
}

char* CCfgParser::operator [] (int i) {
    assert( i >= 0 && i < AtomsNumber );
    return ( LineAtoms[i] );
}

void CCfgParser::ParseLine() {
    struct SLineAtom {
        char *Value;
        SLineAtom *Next;
    };
    SLineAtom *AtomsList, *LastAtom;
    int        i, j;

    assert( AtomsNumber == 0 );

    AtomsList = LastAtom = NULL;
    i = 0;
    while (buf[i] != '\0') {
        while (buf[i] == ' ' || buf[i] == 0x09)
            i++;
        if (buf[i] == ';' || buf[i] == '\0')
            break;
        if (buf[i] == '"') {
            j = i;
            i++;
            while (buf[i] != '"' && buf[i] != '\0')
                i++;
            if (buf[i] == '\0') {
                Error = 1;
                break;
            }
            // buf[i] == '"'
            i++;
        }
        else {
            j = i;
            while (buf[i] != ' ' && buf[i] != 0x09 && buf[i] != ';'
                   && buf[i] != '\0')
                i++;
        }
        // add item into list
        if (AtomsList == NULL) {
            AtomsList = new SLineAtom;
            LastAtom  = AtomsList;
        }
        else {
            LastAtom->Next = new SLineAtom;
            LastAtom       = LastAtom->Next;
        }
        LastAtom->Next = NULL;
        LastAtom->Value = new char[i-j+1];
        memcpy( LastAtom->Value, buf + j, i-j );
        LastAtom->Value[i-j] = '\0';
        AtomsNumber++;
    }
    // create array based on list
    if (Error)
        AtomsNumber = 0;
    if (AtomsNumber > 0) {
        LineAtoms = new char*[AtomsNumber];
        LastAtom  = AtomsList;
        i         = 0;
        while (i < AtomsNumber && LastAtom != NULL) {
            LineAtoms[i] = LastAtom->Value;
            i++;
            LastAtom = LastAtom->Next;
        }
        assert( i == AtomsNumber && LastAtom == NULL );
    }
    // free atoms list
    while (AtomsList != NULL) {
        LastAtom = AtomsList->Next;
        delete AtomsList;
        AtomsList = LastAtom;
    }
}

void CCfgParser::FreeMem() {
    int i;

    if (AtomsNumber > 0) {
        for (i = 0; i < AtomsNumber; i++)
            delete [] LineAtoms[i];
        delete [] LineAtoms;
        AtomsNumber = 0;
    }
}

