#ifndef __KALUGA__MSG__HEADER__
#define __KALUGA__MSG__HEADER__

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "FTNStruc.hpp"
#include "Common.hpp"
#include "msgid.hpp"

// uncomment definition below to substitute Fidonet.org domain
// with Fidonet domain
#define SUBSTITUTE_DOMAIN

enum { SAVE_READ, SAVE_MIDDLE_OF_KLUDGE, SAVE_WRITE_LINE, SAVE_SKIP_LINE };

class CMsg {
private:
    uint16      Attrs;
    uint16      Flags;
    SYSTEMTIME  Time;
    
    int         FMPTExist;
    addr        FMPT;
    
    int         TOPTExist;
    addr        TOPT;
    
    int         INTLExist;
    SFTNAddress INTLFrom;
    SFTNAddress INTLTo;
    
    int         DOMAINExist;
    SFTNAddress DomainKludgeFrom;
    SFTNAddress DomainKludgeTo;
    
    int         ZoneGate;
    SFTNAddress RealTo;
    int         MsgIdExist;
    int         MsgIdDecoded;
    SFTNAddress MsgIdFrom;
    char        MsgIdLine[MAX_KLUDGE_SIZE];
    char        Area[MAX_AREA_SIZE];
    FILE       *f;
    int         MsgInBuf; // message in buffer or not
    int         BodySize; // message body size
    CMsgId      MsgIdHandler;
    int         FlagsExist;
    char        FlagsKludge[MAX_KLUDGE_SIZE];
    char        MsgFileName[MAX_PATH];
    char        MsgFileDir[MAX_PATH];
    // address-treating functions
    void        DecodeAddresses();
    int         ParsePT(char *s, addr &PT);
    int         ParseINTL(char *s);
    int         ParseFlags(char *s);
    int         ParseFTNAddress(char *s, SFTNAddress &Address);
    int         ParseMsgId(char *s);
    int         ParseDOMAIN(char *s);
    // flags-treating fuctions
    void        SetFlag(char *s);
    void        ClearFlag(char *s);
    int         CheckFlag(char *s);
    // Adress-treating functions
    int         FromNet(SFTNAddress &Addr) const; // checks Zone:Net
    int         ToNet(SFTNAddress &Addr)  const;
    int         FromNode(SFTNAddress &Addr)  const; // checks Zone:Net/Node
    int         ToNode(SFTNAddress &Addr)  const;
    int         FromPoint(SFTNAddress &Addr)  const; // checks Zone:Net/Node.Point
    int         ToPoint(SFTNAddress &Addr)  const;
    // Name-treating functions
    int         FromName(char *Name) { return (stricmp(Header.FromUserName,
                                                       Name) == 0); }
    int         ToName(char *Name) { return (stricmp(Header.ToUserName,
                                                     Name) == 0); }
    // Whole message treating functions
    int         SaveMsg(FILE *tmp);
    void        Save();
    int         Move(char *Dir, long &FreeNumber);
    void        Bounce( uint16 BounceAttrs, uint16 BounceFlags,
                        char *BounceTplFileName,
                        SFTNAddress &BounceAddr );
    // user-defined fields and functions
    int         FromUs();
    int         ToUs();
public:
    long        FreeMsgNumber;
    SMsgHeader  Header;
    SFTNAddress From, To;
    CMsg();
    ~CMsg();
    int  Read(char *MsgFName);
    void SetFreeMsgNumber(long Number) { FreeMsgNumber = Number; }
    // these two are with domains (Zone:Net/Node.Point@Domain)
    int  FromAddr(SFTNAddress &Addr);
    int  ToAddr(SFTNAddress &Addr);
    int  Delete();
    void Reply( uint16 ReplyAttrs, uint16 ReplyFlags, char *Subject,
                SStringList *BodyLines );
    void SaveAsReceived();
};

#endif